#!/usr/bin/env perl

=head1 NAME

command.pl - A command-line program with POD

=head1 SYNOPSIS

    command.pl -h
    command.pl -v

=head1 DESCRIPTION

This is a command whose POD should be available from L<command>.
