package
    My::Internal;

=head1 NAME

My::Internal - An internal module to link to

=head1 SYNOPSIS

    my $int = My::Internal->new;

=head1 DESCRIPTION

This is an internal module to test that linking works. Let's link to L<My> just to
make sure that it works.
