
/************************************************
 * Basic style
 ************************************************/

body {
    /* Raleway looks terrible, and in some cases is hard to read */
    font-family: "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

h1 {
    font-size: 3.5em;
}

h2 {
    font-size: 2.75em;
}

h3 {
    font-size: 2.25em;
}

h4 {
    font-size: 1.75em;
}

h5 {
    font-size: 1.3em;
}

code {
    /* Only pre should set nowrap */
    white-space: normal;
}

/* Fix <p> inside <li> leaving the list bullet on its own line in Firefox */
ul {
    list-style: circle outside;
    margin-left: 1.4em;
}
ol {
    list-style: decimal outside;
    margin-left: 1.4em;
}

ul ul, ul ol, ol ul, ol ol {
    /* Fix each successive layer of lists getting smaller and smaller.
     * After three layers of lists, it's too small to read well
     */
    font-size: 100%;
}

.button.disabled {
    cursor: not-allowed;
}

main header h1 {
    border-bottom: 1px solid #045578;
}

main header h1 a {
    text-decoration: none;
}

header aside {
    margin-top: 0;
    font-size: 12pt;
}

/** Add a line when using the "Continue reading..." link */
section:target {
    padding-top: 0.5em;
    border-top: 2px dashed #777;
}

/** Don't allow <pre> to push too much */
pre {
    max-width: 100%;
    overflow-x: scroll;
}

/**
 * Don't allow main content images to push things around
 */
main img {
    max-width: 100%;
}

a[rel=external]::after {
    content: '\f08e';
    padding-left: 0.2em;
    font-family: "FontAwesome";
    display: inline-block;
    font-size: smaller;
    vertical-align: top;
}

/************************************************
 * Classes
 ************************************************/

ol.bare, ul.bare {
    list-style-type: none;
    margin-left: 0;
}

ol.list-inline, ul.list-inline {
    margin: 0;
}

ol.list-inline li, ul.list-inline li {
    display: inline-block;
    margin: 0;
}

img.u-pull-left, img.u-pull-right {
    /* Give some padding so the text isn't too close */
    padding: 0.2em;
}

/************************************************
 * Grid
 ************************************************/

.column, .columns {
    margin-left: 2%;
    padding-left: 1%;
    padding-right: 1%;
}

/************************************************
 * Widgets
 ************************************************/

/** Navbar widget */
.navbar {
    overflow: hidden;
    border-bottom: 1px solid #bbb;
    color: #777;
    background-color: white;
    margin-bottom: 1em;
}

.navbar a {
    color: #777;
    text-decoration: none;
}

.navbar a:hover {
    color: #1EAEDB;
}

.navbar .brand {
    font-size: 18px;
    line-height: 20px;
    margin-right: 0.5em;
}

.navbar ul {
    list-style: none;
    display: inline-block;
    margin: 1em 0;
}

.navbar li {
    display: inline-block;
    padding: 0 1em;
    margin: 0;
}

/** Sidebar widget */
.sidebar h1 {
    font-size: 2.25em;
}

.sidebar h2 {
    font-size: 1.75em;
}

.sidebar h3 {
    font-size: 1.3em;
}

.sidebar h4 {
    font-size: 1.1em;
}

.sidebar h5 {
    font-size: 1em;
}

.sidebar {
    font-size: 0.8em;
}

/** Pager widget */
ul.pager {
    list-style: none;
}

.pager .prev {
    float: left;
}

.pager .next {
    float: right;
}

/** Footer */
footer {
    margin-top: 1em;
    padding-bottom: 0.5em;
    font-size: 0.7em;
}

footer .tagline {
    font-size: smaller;
    line-height: 1.3;
    text-align: right;
}

/** Crumbtrail widget */

.crumbtrail ul {
    display: inline-block;
    list-style: none;
    margin: 0;
}

.crumbtrail li {
    display: inline-block;
}

.crumbtrail li::before {
    content: '::';
    display: inline-block;
}

.crumbtrail li:first-child::before {
    content: '';
}

/************************************************
 * Applications
 */

/** Blog */
.tags {
    float: right;
}

